@@ -100,8 +100,10 @@ module Control.Monad.Logger
 #endif
     -- * utilities for defining your own loggers
     , defaultLogStr
+    -- $locDocs
     , Loc (..)
     , defaultLoc
+    , defaultOutput
     ) where
 
 #if WITH_TEMPLATE_HASKELL
@@ -178,6 +180,22 @@ data LogLevel = LevelDebug | LevelInfo | LevelWarn | LevelError | LevelOther Tex
 
 type LogSource = Text
 
+-- $locDocs
+--
+-- === Loc
+--
+-- When @monad-logger@ is compiled with the @template_haskell@ flag set to true (the default), the 'Loc' below is a re-export from the @template-haskell@ package.
+-- When the flag is false, the 'Loc' below is a copy of that data structure defined in @monad-logger@ itself.
+--
+-- If you are making a library that:
+--
+-- * Uses @monad-logger@
+-- * Uses 'Loc' in a type signature
+-- * But doesn't need to depend on @template-haskell@ for other reasons
+--
+-- You can import 'Loc' directly from this package, instead of adding an dependency on @template-haskell@ and importing from there.
+-- This allows users to compile your package in environments that don't support @template-haskell@.
+
 #if WITH_TEMPLATE_HASKELL
 
 instance Lift LogLevel where
@@ -416,6 +434,12 @@ instance MonadUnliftIO m => MonadUnliftIO (NoLoggingT m) where
     return (UnliftIO (unliftIO u . runNoLoggingT))
 #endif
 
+instance (Applicative m, Semigroup a) => Semigroup (NoLoggingT m a) where
+  (<>) = liftA2 (<>)
+
+instance (Applicative m, Monoid a) => Monoid (NoLoggingT m a) where
+  mempty = pure mempty
+
 -- | @since 0.3.32
 type LogLine = (Loc, LogSource, LogLevel, LogStr)
 
@@ -536,6 +560,12 @@ instance MonadMask m => MonadMask (WriterLoggingT m) where
           return (y, appendDList w1 w2))
 #endif
 
+instance (Applicative m, Semigroup a) => Semigroup (WriterLoggingT m a) where
+  (<>) = liftA2 (<>)
+
+instance (Applicative m, Monoid a) => Monoid (WriterLoggingT m a) where
+  mempty = pure mempty
+
 -- | Monad transformer that adds a new logging function.
 --
 -- @since 0.2.2
@@ -647,6 +677,24 @@ instance MonadUnliftIO m => MonadUnliftIO (LoggingT m) where
     return (UnliftIO (unliftIO u . flip runLoggingT f))
 #endif
 
+instance (Applicative m, Semigroup a) => Semigroup (LoggingT m a) where
+  (<>) = liftA2 (<>)
+
+instance (Applicative m, Monoid a) => Monoid (LoggingT m a) where
+  mempty = pure mempty
+
+-- | A default implementation of 'monadLoggerLog' that accepts a file
+-- handle as the first argument.
+--
+-- This is used in the definition of 'runStdoutLoggingT':
+--
+-- @
+-- 'runStdoutLoggingT' :: 'MonadIO' m => 'LoggingT' m a -> m a
+-- 'runStdoutLoggingT' action =
+--     'runLoggingT' action ('defaultOutput' 'stdout')
+-- @
+--
+-- @since 0.3.36
 defaultOutput :: Handle
               -> Loc
               -> LogSource
@@ -657,13 +705,14 @@ defaultOutput h loc src level msg =
     S8.hPutStr h ls
   where
     ls = defaultLogStrBS loc src level msg
+
 defaultLogStrBS :: Loc
                 -> LogSource
                 -> LogLevel
                 -> LogStr
                 -> S8.ByteString
 defaultLogStrBS a b c d =
-    toBS $ defaultLogStr a b c d
+    fromLogStr $ defaultLogStr a b c d
   where
     toBS = fromLogStr
 
@@ -872,19 +921,19 @@ logErrorN = logWithoutLoc "" LevelError
 logOtherN :: MonadLogger m => LogLevel -> Text -> m ()
 logOtherN = logWithoutLoc ""
 
-logDebugNS :: MonadLogger m => Text -> Text -> m ()
+logDebugNS :: MonadLogger m => LogSource -> Text -> m ()
 logDebugNS src = logWithoutLoc src LevelDebug
 
-logInfoNS :: MonadLogger m => Text -> Text -> m ()
+logInfoNS :: MonadLogger m => LogSource -> Text -> m ()
 logInfoNS src = logWithoutLoc src LevelInfo
 
-logWarnNS :: MonadLogger m => Text -> Text -> m ()
+logWarnNS :: MonadLogger m => LogSource -> Text -> m ()
 logWarnNS src = logWithoutLoc src LevelWarn
 
-logErrorNS :: MonadLogger m => Text -> Text -> m ()
+logErrorNS :: MonadLogger m => LogSource -> Text -> m ()
 logErrorNS src = logWithoutLoc src LevelError
 
-logOtherNS :: MonadLogger m => Text -> LogLevel -> Text -> m ()
+logOtherNS :: MonadLogger m => LogSource -> LogLevel -> Text -> m ()
 logOtherNS = logWithoutLoc
 
 #if WITH_CALLSTACK
